@@ -51,8 +51,10 @@ show_file (char *filename)
     }
 
   find_file (filename);
-  cur_bp->flags = BFLAG_READONLY | BFLAG_NOSAVE | BFLAG_NEEDNAME
-    | BFLAG_NOUNDO;
+  set_buffer_readonly (cur_bp, true);
+  set_buffer_noundo (cur_bp, true);
+  set_buffer_needname (cur_bp, true);
+  set_buffer_nosave (cur_bp, true);
 
   return leT;
 }
@@ -138,7 +140,7 @@ Display the full documentation of a variable.
   else
     {
       char *defval;
-      const char *doc = get_variable_doc (cur_bp, name, &defval);
+      const char *doc = get_variable_doc (name, &defval);
       if (doc == NULL)
         ok = leNIL;
       else
@@ -148,7 +150,7 @@ Display the full documentation of a variable.
           astr_afmt (bufname, "*Help: variable `%s'*", name);
           write_temp_buffer (astr_cstr (bufname), true,
                              write_variable_description,
-                             name, defval, get_variable (name), doc);
+                             name, get_variable (name), doc);
           astr_delete (bufname);
         }
     }
