@@ -21,7 +21,6 @@
 
 #include "config.h"
 
-#include <ctype.h>
 #include <limits.h>
 #include <locale.h>
 #include <stdarg.h>
@@ -76,7 +75,7 @@ loop (void)
         last_uniarg = 1;
 
       if (last_command () != F_undo)
-        cur_bp->next_undop = cur_bp->last_undop;
+        set_buffer_next_undop (cur_bp, get_buffer_last_undop (cur_bp));
 
       lastflag = thisflag;
     }
@@ -88,7 +87,7 @@ static char about_splash_str[] = "\
 " ZILE_COPYRIGHT_STRING "\n\
 \n\
 Type `C-x C-c' to exit " PACKAGE_NAME ".\n\
-Type `C-x u; to undo changes.\n\
+Type `C-x u' to undo changes.\n\
 Type `C-g' at any time to quit the current operation.\n\
 \n\
 `C-x' means hold the CTRL key while typing the character `x'.\n\
@@ -112,6 +111,20 @@ about_screen (void)
     }
 }
 
+static void
+execute_functions (gl_list_t funcs)
+{
+  size_t i;
+  for (i = 0; i < gl_list_size (funcs); i++)
+    {
+      char *func = (char *) gl_list_get_at (funcs, i);
+      term_redisplay ();
+      if (!execute_function (func, 1))
+        minibuf_error ("Function `%s' not defined", func);
+      lastflag |= FLAG_NEED_RESYNC;
+    }
+}
+
 static void
 load_files (gl_list_t files)
 {
@@ -137,10 +150,10 @@ setup_main_screen (int argc)
   Buffer *bp, *last_bp = NULL;
   int c = 0;
 
-  for (bp = head_bp; bp; bp = bp->next)
+  for (bp = head_bp; bp; bp = get_buffer_next (bp))
     {
       /* Last buffer that isn't *scratch*. */
-      if (bp->next && !bp->next->next)
+      if (get_buffer_next (bp) && !get_buffer_next (get_buffer_next (bp)))
         last_bp = bp;
       c++;
     }
@@ -173,7 +186,7 @@ then enter the text in that file's own buffer.\n\
             }
 
           undo_nosave = false;
-          cur_bp->flags &= ~BFLAG_MODIFIED;
+          set_buffer_modified (cur_bp, false);
           lastflag |= FLAG_NEED_RESYNC;
         }
     }
@@ -229,7 +242,10 @@ main (int argc, char **argv)
   int qflag = false;
   gl_list_t l_args = gl_list_create_empty (GL_LINKED_LIST,
                                            NULL, NULL, NULL, false);
+  gl_list_t f_args = gl_list_create_empty (GL_LINKED_LIST,
+                                           NULL, NULL, NULL, false);
   size_t line;
+  Buffer *scratch_bp;
 
   /* Set prog_name to executable name, if available */
   if (argv[0])
@@ -249,7 +265,7 @@ main (int argc, char **argv)
       char *buf, *shortopt;
 
       /* Leading : so as to return ':' for a missing arg, not '?' */
-      c = getopt_long (argc, argv, ":l:q", longopts, &longindex);
+      c = getopt_long (argc, argv, ":f:l:q", longopts, &longindex);
 
       if (c == -1)
         break;
@@ -261,6 +277,12 @@ main (int argc, char **argv)
                    prog_name, argv[this_optind]);
           exit (1);
         }
+      else if (c == 'q')
+        longindex = 0;
+      else if (c == 'f')
+        longindex = 1;
+      else if (c == 'l')
+        longindex = 2;
 
       switch (longindex)
         {
@@ -268,9 +290,12 @@ main (int argc, char **argv)
           qflag = true;
           break;
         case 1:
-          gl_list_add_last (l_args, (void *) optarg);
+          gl_list_add_last (f_args, (void *) optarg);
           break;
         case 2:
+          gl_list_add_last (l_args, (void *) optarg);
+          break;
+        case 3:
           printf ("Usage: %s [OPTION-OR-FILENAME]...\n"
                   "\n"
                   "Run " PACKAGE_NAME ", the lightweight Emacs clone.\n"
@@ -293,7 +318,7 @@ main (int argc, char **argv)
           printf ("\n"
                   "Report bugs to " PACKAGE_BUGREPORT ".\n");
           exit (0);
-        case 3:
+        case 4:
           printf (ZILE_VERSION_STRING "\n"
                   ZILE_COPYRIGHT_STRING "\n"
                   "GNU " PACKAGE_NAME " comes with ABSOLUTELY NO WARRANTY.\n"
@@ -318,6 +343,7 @@ main (int argc, char **argv)
 
   /* Create the `*scratch*' buffer. */
   create_first_window ();
+  scratch_bp = cur_bp;
   term_redisplay ();
 
   /* Read settings after creating *scratch* buffer so that any
@@ -333,9 +359,6 @@ main (int argc, char **argv)
         }
     }
 
-  /* Reinitialise the scratch buffer to catch settings */
-  init_buffer (cur_bp);
-
   /* Open files given on the command line */
   for (line = 1; *argv; argv++)
     {
@@ -352,11 +375,23 @@ main (int argc, char **argv)
 
   /* Show the splash screen only if no files and no Lisp expression
      or load file is specified on the command line. */
-  if (minibuf_contents == NULL && argc == 0 && gl_list_size (l_args) == 0)
+  if (minibuf_contents == NULL && argc == 0 &&
+      gl_list_size (f_args) == 0 &&
+      gl_list_size (l_args) == 0)
     about_screen ();
-
   setup_main_screen (argc);
 
+  /* Load Lisp files given on the command line. */
+  load_files (l_args);
+  gl_list_free (l_args);
+
+  /* Reinitialise the scratch buffer to catch settings */
+  init_buffer (scratch_bp);
+
+  /* Run command-line functions. */
+  execute_functions (f_args);
+  gl_list_free (f_args);
+
   /* Refresh minibuffer in case there's an error that couldn't be
      written during startup */
   if (minibuf_contents != NULL)
@@ -367,9 +402,6 @@ main (int argc, char **argv)
       free (buf);
     }
 
-  load_files (l_args);
-  gl_list_free (l_args);
-
   /* Run the main loop. */
   if (!(thisflag & FLAG_QUIT))
     loop ();
@@ -385,6 +417,7 @@ main (int argc, char **argv)
   free_lisp ();
 
   /* Free all the memory allocated. */
+  free_search ();
   free_kill_ring ();
   free_registers ();
   free_macros ();
