@@ -63,8 +63,6 @@ var_free (void *v)
 {
   var_entry *p = (var_entry *) v;
   free ((char *) p->var);
-  if (p->defval != p->val)
-    free ((char *) p->defval);
   free ((char *) p->val);
   free (p);
 }
@@ -74,7 +72,8 @@ init_builtin_var (const char *var, const char *defval, bool local, const char *d
 {
   var_entry *p = XZALLOC (var_entry);
   p->var = xstrdup (var);
-  p->defval = p->val = xstrdup (defval);
+  p->defval = defval;
+  p->val = xstrdup (defval);
   p->local = local;
   p->doc = doc;
   hash_insert (main_vars, p);
@@ -97,41 +96,44 @@ init_variables (void)
 #undef X
 }
 
-static void
-set_variable_in_list (Hash_table *var_list, const char *var, const char *val)
+void
+set_variable (const char *var, const char *val)
 {
+  Hash_table *var_list;
+  struct var_entry *ent, *key = XZALLOC (var_entry);
   var_entry *p = XZALLOC (var_entry), *q;
 
-  /* Insert variable if it doesn't already exist */
+  /* Find whether variable is buffer-local when set, and if needed
+     create a buffer-local variable list. */
+  key->var = var;
+  ent = hash_lookup (main_vars, key);
+  free (key);
+  if (ent && ent->local && get_buffer_vars (cur_bp) == NULL)
+    set_buffer_vars (cur_bp, new_varlist ());
+  var_list = (ent && ent->local) ? get_buffer_vars (cur_bp) : main_vars;
+
+  /* Insert variable if it doesn't already exist. */
   p->var = xstrdup (var);
   q = hash_insert (var_list, p);
 
   /* Update value */
+  free ((char *) q->val);
   q->val = xstrdup (val);
 
-  /* If variable is new, initialise other fields */
+  /* If variable is new, initialise other fields. */
   if (q == p)
     {
-      p->defval = xstrdup (val);
-      p->local = false;
-      p->doc = "";
+      if (var_list == main_vars)
+        {
+          p->defval = xstrdup (val);
+          p->local = false;
+          p->doc = "";
+        }
     }
-  else
+  if (q != p)
     var_free (p);
 }
 
-void
-set_variable (const char *var, const char *val)
-{
-  struct var_entry *ent, *key = XZALLOC (var_entry);
-  key->var = var;
-  ent = hash_lookup (main_vars, key);
-  free (key);
-  if (ent && ent->local && cur_bp->vars == NULL)
-    cur_bp->vars = new_varlist ();
-  set_variable_in_list ((ent && ent->local) ? cur_bp->vars : main_vars, var, val);
-}
-
 void
 free_variables (void)
 {
@@ -145,8 +147,8 @@ get_variable_entry (Buffer * bp, const char *var)
 
   key->var = var;
 
-  if (bp && bp->vars)
-    p = hash_lookup (bp->vars, key);
+  if (bp && get_buffer_vars (bp))
+    p = hash_lookup (get_buffer_vars (bp), key);
 
   if (p == NULL)
     p = hash_lookup (main_vars, key);
@@ -157,9 +159,9 @@ get_variable_entry (Buffer * bp, const char *var)
 }
 
 const char *
-get_variable_doc (Buffer * bp, const char *var, char **defval)
+get_variable_doc (const char *var, char **defval)
 {
-  var_entry *p = get_variable_entry (bp, var);
+  var_entry *p = get_variable_entry (NULL, var);
   if (p != NULL)
     {
       *defval = (char *) p->defval;
@@ -222,7 +224,7 @@ minibuf_read_variable_name (char *fmt, ...)
        p != NULL;
        p = hash_get_next (main_vars, p))
     {
-      gl_sortedlist_add (cp->completions, completion_strcmp,
+      gl_sortedlist_add (get_completion_completions (cp), completion_strcmp,
                          xstrdup (p->var));
     }
 
